/**
 * Basic imitation of jQuery's ajax function.
 */
function Ajax(options) {
    var default_options = {
        method: 'POST',
        headers: {"X-Requested-With": "XMLHttpRequest",},
        success: function(){},
        error: function(){},
    };
    var _opt = {};
    Object.assign(_opt, default_options, options);

    var _callback = function(opt) {
        fetch(opt.url, {
            credentials: "same-origin",
            method: opt.method,
            headers: new Headers(opt.headers),
            body: opt.data,
        }).then(function (response) {
            return response.json().then(function(json) {
                return {
                    'status': response.status,
                    'payload': json,
                };
            });
        }).then(function (response) {
            if (response.status == 200) {
                try {
                    opt.success(response.payload);
                } catch (e) {
                    console.error("Error in your callback function", e);
                }
            } else if (response.status == 403) {
                if ('error' in response.payload) {
                    alert(response.payload.error);
                }
                if ('redirect' in response.payload) {
                    location.assign(response.payload.redirect);
                } else {
                    location.reload();
                }
            } else {
                opt.error();
                return undefined;
            }
        }).catch(function(error) {
            console.error('There has been a problem with your fetch operation:', error);
            if (error.status == 429) {
                // Too Many Requests
                console.error(error.statusText);
            }
        });
    };


    if (_opt.route_name !== undefined) {
        route_path(_opt.route_name, _opt.route_params, function(url) {
            delete _opt.route_name;
            delete _opt.route_params;
            _opt.url = url;
            _callback(_opt);
        });
    } else {
        console.debug("opt");
        _callback(_opt);
    }
}


/**
 * Proxy function for pyramid's request.route_path() function.
 * 
 * Typical usage:
 * <pre>
 * window.open(route_url('product', {id:5}));
 * </pre>
 * 
 * @param route_name
 * @param params
 * @returns
 */
function route_path(route_name, params, callback) {
    if (!sessionStorage.routes) {
        sessionStorage.routes = JSON.stringify({});
    }
    var routes = JSON.parse(sessionStorage.routes);
    var fake_params = {};
    if (params) {
        var keys = Object.keys(params);
        var k,v;
        for (var i = 0; i < keys.length; i++) {
            k = keys[i];
            fake_params[k] = k.toUpperCase();
        }
    }
    if (!routes[route_name]) {
        Ajax({
            url: '/__route_path__',
            data: JSON.stringify({ route_name: route_name, params: fake_params }),
            method: 'POST',
            success: function (response) {
                routes[route_name] = response.route_path;
                sessionStorage.routes = JSON.stringify(routes);
            },
        });
    }
    var result = routes[route_name];
    if (params) {
        var k, v, fake_val;
        for (var i = 0; i < keys.length; i++) {
            k = keys[i];
            v = params[k];
            fake_val = fake_params[k];
            result = result.replace(fake_val, v);
        }
    }
    if (callback !== undefined) {
        callback(result);
    } else {
        return result;
    }
}
