from pyramid.view import view_config
from pyramid.response import Response
from pyramid.httpexceptions import HTTPFound, HTTPNotFound, HTTPInternalServerError
from pyramid.security import NO_PERMISSION_REQUIRED, Authenticated
from sqlalchemy.exc import SQLAlchemyError

import logging
log = logging.getLogger(__name__)

from .. import models
import base64
from pprint import pprint

import pbaccount.api.core as coreapi


@view_config(route_name='login', renderer='', permission=NO_PERMISSION_REQUIRED)
def login(request):
    if request.method == "POST":
        principals = [Authenticated]
        if request.params.get("uid") == "eeadmin":
            principals = [Authenticated, "investigator", "caregiver:dp", "caregiver:tb", "caregiver:sz"]
        else:
            principals = [Authenticated, "caregiver"]
        request.session['userdata'] = {
            "userid": request.params.get("uid"),
            "principals": principals,
        }

        return HTTPFound(location=request.route_path("home"))
    return Response("""<html>
<head>
  <title>login</title>
</head>
<body>
<form method="POST">
<input type="text" name="uid" placeholder="username" />
<button type="submit">OK</button>
</form>
<body>
</html>""")


@view_config(route_name='route_path', renderer='json', permission=NO_PERMISSION_REQUIRED)
def route_path(request):
    '''Special view to make pyramid's ``request.route_path()`` callable
    from javascript code using ajax.'''
    data = request.json_body
    route_name = data.get('route_name')
    params = data.get('params', {})
    return {
        'route_path': request.route_path(route_name, **params),
    }


@view_config(route_name='home', renderer='pbaccount:templates/home.html.jinja2', permission=Authenticated)
def home_view(request):
    account = coreapi.get_account(request.userdata.get("userid"))
    if account.get("jpegPhoto") and len(account["jpegPhoto"]):
        account["jpegPhoto"] = base64.b64encode(account.get("jpegPhoto")).decode()
    return {
        "user": account,
    }


@view_config(route_name='caregiver.course', renderer='pbaccount:templates/caregiver_course.html.jinja2', permission=Authenticated)
def caregiver_course(request):
    """"""
    return {}

@view_config(route_name='invitation.list', renderer='pbaccount:templates/invitation.list.html.jinja2', permission="investigator")
def invitation_list(request):
    return {
        "invitations": coreapi.list_invitations(),
    }


@view_config(route_name='invitation.new', renderer='pbaccount:templates/invitation.new.html.jinja2', permission="investigator")
def invitation_new(request):
    if request.method == 'POST':
        try:
            coreapi.create_invitation(request.params.get("email"))
        except Exception as e:
            request.session.flash(str(e))
        return HTTPFound(location=request.route_path("invitation.list"))
    else:
        return {}


@view_config(route_name='invitation', renderer='json', request_method="DELETE", permission="investigator")
def invitation_remove(request):
    coreapi.remove_invitation(request.matchdict.get("token"))
    if request.is_xhr:
        return {}
    return HTTPFound(location=request.route_path("invitation.list"))


@view_config(route_name='account.register', renderer='pbaccount:templates/account.register.html.jinja2', permission=NO_PERMISSION_REQUIRED)
def account_register(request):
    invitation_token = request.matchdict.get("token")
    invit = coreapi.find_invitation(invitation_token)
    if invit is None:
        return HTTPNotFound("Invitation inexistante")
    if request.method == 'POST':
        nickname = request.params.get("nickname")
        password = request.params.get("password")
        existing_account = coreapi.get_account(nickname)
        if existing_account is not None:
            return HTTPInternalServerError("Pseudonyme déjà utilisé")
        coreapi.create_account(nickname, invit.email, password)
        coreapi.remove_invitation(invitation_token)

        coreapi.profile_update_item(nickname, "avatar", request.params.get("avatar").file.read())
        coreapi.profile_update_item(nickname, "mobile", request.params.get("mobile"))
        coreapi.profile_update_item(nickname, "description", request.params.get("description"))
        coreapi.profile_update_item(nickname, "experience", request.params.get("experience"))
        coreapi.profile_update_item(nickname, "gender", request.params.get("gender"))
        coreapi.profile_update_item(nickname, "birthdate", request.params.get("birthdate"))
        coreapi.profile_update_item(nickname, "department", request.params.get("department"))

        pathologies = request.params.getall("pathology")
        relationships = request.params.getall("relationship")
        coreapi.profile_update_item(nickname, "patients", ["[pathology={}][relationship={}]".format(p, r) for p,r in tuple(zip(pathologies, relationships))])

        return HTTPFound(location=request.route_path("account.welcome", nickname=nickname))
    else:
        invitation_token = request.matchdict.get("token")
        invit = coreapi.find_invitation(invitation_token)
        return {
            "user": {
                "mail": invit.email,
            }
        }


@view_config(route_name='account.welcome', renderer='pbaccount:templates/account.welcome.html.jinja2', permission=NO_PERMISSION_REQUIRED)
def account_welcome(request):
    return {
        "nickname": request.matchdict.get("nickname")
    }

@view_config(route_name='account.view', renderer='pbaccount:templates/account.view.html.jinja2', permission="investigator")
def account_view(request):
    uid = request.matchdict.get("nickname")
    account = coreapi.get_account(uid)
    if account.get("jpegPhoto") and len(account["jpegPhoto"]):
        account["jpegPhoto"] = base64.b64encode(account.get("jpegPhoto")).decode()
    return {
        "user": account,
    }


@view_config(route_name='account.list', renderer='pbaccount:templates/account.list.html.jinja2', permission="investigator")
def account_list(request):
    return {
        "users": coreapi.list_accounts(),
    }


@view_config(route_name='patient.add', renderer='json', permission=Authenticated)
def patient_add(request):
    coreapi.patient_add(
        request.params.get("uid"),
        request.params.get("pathology"),
        request.params.get("relationship"),
    )
    return HTTPFound(request.referer)

